@@ -51,6 +51,8 @@
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-volume.h>
+#include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include "nautilus-file-changes-queue.h"
 #include "nautilus-file-private.h"
 #include "nautilus-desktop-icon-file.h"
@@ -575,22 +577,17 @@ handle_transfer_ok (const GnomeVFSXferProgressInfo *progress_info,
 		/* fall through */
 	case GNOME_VFS_XFER_PHASE_COPYING:
 		if (transfer_info->progress_dialog != NULL) {
-			if (progress_info->bytes_copied == 0) {
-				progress_dialog_set_to_from_item_text
-					(transfer_info->progress_dialog,
-					 transfer_info->progress_verb,
-					 progress_info->source_name,
-					 progress_info->target_name,
-					 progress_info->file_index,
-					 progress_info->file_size);
-			} else {
-				nautilus_file_operations_progress_update_sizes
-					(transfer_info->progress_dialog,
-					 MIN (progress_info->bytes_copied, 
-					      progress_info->bytes_total),
-					 MIN (progress_info->total_bytes_copied,
-					      progress_info->bytes_total));
-			}
+			progress_dialog_set_to_from_item_text (transfer_info->progress_dialog,
+							       transfer_info->progress_verb,
+							       progress_info->source_name,
+							       progress_info->target_name,
+							       progress_info->file_index,
+							       progress_info->file_size);
+			nautilus_file_operations_progress_update_sizes (transfer_info->progress_dialog,
+									MIN (progress_info->bytes_copied, 
+									     progress_info->bytes_total),
+									MIN (progress_info->total_bytes_copied,
+									     progress_info->bytes_total));
 		}
 		return 1;
 
@@ -1123,7 +1120,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 		           TransferInfo *transfer_info)
 {
 	int result;
-	char *text, *primary_text, *secondary_text, *formatted_name;
+	char *text, *primary_text, *secondary_text, *formatted_name, *base_name;
+	GnomeVFSURI *file_uri, *parent_uri;
 	gboolean is_merge, target_is_dir;
 
 	nautilus_file_operations_progress_pause_timeout (transfer_info->progress_dialog);	
@@ -1164,26 +1162,43 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 	}
 	
 	/* transfer conflict, prompt the user to replace or skip */
-	formatted_name = format_and_ellipsize_uri_for_dialog (
-		parent_for_error_dialog (transfer_info), progress_info->target_name);
+	file_uri = gnome_vfs_uri_new (progress_info->target_name);
+	base_name = gnome_vfs_uri_extract_short_path_name (file_uri);
+	formatted_name = gnome_vfs_unescape_string_for_display (base_name);
 
 	target_is_dir = is_directory (progress_info->target_name);
 	if (target_is_dir) {
-		text = g_strdup_printf (_("The folder \"%s\" already exists.  Would you like to replace it?"), 
+		text = g_strdup_printf (_("A folder named \"%s\" already exists.  Do you want to replace it?"), 
 					formatted_name);
 	} else {
-		text = g_strdup_printf (_("The file \"%s\" already exists.  Would you like to replace it?"), 
+		text = g_strdup_printf (_("A file named \"%s\" already exists.  Do you want to replace it?"), 
 					formatted_name);
 	}
+	g_free (base_name);
 	g_free (formatted_name);
 
+	if (gnome_vfs_uri_has_parent (file_uri)) {
+		parent_uri = gnome_vfs_uri_get_parent (file_uri);
+		base_name = gnome_vfs_uri_extract_short_path_name (parent_uri);
+		gnome_vfs_uri_unref (parent_uri);	
+	} else {
+		base_name = gnome_vfs_uri_extract_dirname (file_uri);
+	}
+	
+	formatted_name = gnome_vfs_unescape_string_for_display (base_name);
+	
 	is_merge =  target_is_dir && is_directory (progress_info->source_name);
 
 	if (is_merge) {
-		secondary_text = _("If you replace the existing folder, any files in it that conflict with the files being copied will be overwritten.");
+		secondary_text = g_strdup_printf (_("The folder already exists in \"%s\".  Replacing it will overwrite any files in the folder that conflict with the files being copied."), 
+		                                  formatted_name);
 	} else {
-		secondary_text = _("If you replace an existing file, its contents will be overwritten.");
+		secondary_text = g_strdup_printf (_("The file already exists in \"%s\".  Replacing it will overwrite its contents."), 
+		                                  formatted_name);
 	}
+	gnome_vfs_uri_unref (file_uri);
+	g_free (formatted_name);
+	g_free (base_name);
 	
 	if (progress_info->duplicate_count == 1) {
 		/* we are going to only get one duplicate alert, don't offer
@@ -1192,11 +1207,12 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 		result = eel_run_simple_dialog 
 			(parent_for_error_dialog (transfer_info),
 			 TRUE,
-			 GTK_MESSAGE_QUESTION, 
+			 GTK_MESSAGE_WARNING, 
 			 text, 
 			 secondary_text, 
 			 _("_Skip"), _("_Replace"), NULL);
 		g_free (text);	 
+		g_free (secondary_text);
 
 		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
 					 
@@ -1211,10 +1227,11 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 		}
 	} else {
 		result = eel_run_simple_dialog
-			(parent_for_error_dialog (transfer_info), TRUE, GTK_MESSAGE_QUESTION, text, 
+			(parent_for_error_dialog (transfer_info), TRUE, GTK_MESSAGE_WARNING, text, 
 			 secondary_text, 
 			 _("S_kip All"), _("Replace _All"), _("_Skip"), _("_Replace"), NULL);
 		g_free (text);
+		g_free (secondary_text);
 
 		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
 
@@ -1256,6 +1273,18 @@ get_link_name (char *name, int count)
 
 	unescaped_name = g_filename_to_utf8 (unescaped_tmp_name, -1,
 					     NULL, NULL, NULL);
+
+	if (!unescaped_name) {
+		/* Couldn't convert to utf8 - probably
+		 * G_BROKEN_FILENAMES not set when it should be.
+		 * Try converting from the locale */
+		unescaped_name = g_locale_to_utf8 (unescaped_tmp_name, -1, NULL, NULL, NULL);	
+
+		if (!unescaped_name) {
+			unescaped_name = eel_make_valid_utf8 (unescaped_tmp_name);
+		}
+	}
+
 	g_free (unescaped_tmp_name);
 
 	if (count < 1) {
@@ -1869,6 +1898,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	gboolean target_is_mapping;
 	gboolean have_nonmapping_source;
 	gboolean have_nonlocal_source;
+	gboolean have_readonly_source;
 	
 	IconPositionIterator *icon_position_iterator;
 
@@ -1902,6 +1932,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	target_uri_list = NULL;
 	have_nonlocal_source = FALSE;
 	have_nonmapping_source = FALSE;
+	have_readonly_source = FALSE;
 	duplicate = copy_action != GDK_ACTION_MOVE;
 	for (p = item_uris; p != NULL; p = p->next) {
 		/* Filter out special Nautilus link files */
@@ -1926,7 +1957,31 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		if (strcmp (source_uri->method_string, "burn") != 0) {
 			have_nonmapping_source = TRUE;
 		}
-			
+
+		if (!have_readonly_source) {
+			GnomeVFSVolume *volume;
+			char *text_uri, *path;
+
+			text_uri = gnome_vfs_uri_to_string (source_uri, GNOME_VFS_URI_HIDE_NONE);
+
+			path = gnome_vfs_get_local_path_from_uri (text_uri);
+
+			volume = NULL;
+			if (path != NULL) {
+				/* TODO should we resolve symlinks ourselves, or should
+				 * gnome_vfs_volume_monitor_get_volume_for_path use lstat? */
+				volume = gnome_vfs_volume_monitor_get_volume_for_path (gnome_vfs_get_volume_monitor (), path);
+			}
+
+			if (volume != NULL && gnome_vfs_volume_is_read_only (volume)) {
+				have_readonly_source = TRUE;
+			}
+
+			gnome_vfs_volume_unref (volume);
+			g_free (path);
+			g_free (text_uri);
+		}
+
 		/* Note: this could be null if we're e.g. copying the top level file of a web site */
 		source_dir_uri = gnome_vfs_uri_get_parent (source_uri);
 		target_uri = NULL;
@@ -1984,6 +2039,10 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		move_options |= GNOME_VFS_XFER_USE_UNIQUE_NAMES;
 	}
 
+	if (have_readonly_source) {
+		move_options |= GNOME_VFS_XFER_TARGET_DEFAULT_PERMS;
+	}
+
 	/* List may be NULL if we filtered all items out */
 	if (source_uri_list == NULL) {
 		if (target_dir_uri != NULL) {
