@@ -40,7 +40,7 @@
 #include <eel/eel-string.h>
 #include <eel/eel-xml-extensions.h>
 #include <eel/eel-vfs-extensions.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libxml/parser.h>
@@ -51,24 +51,6 @@
 #define NAUTILUS_LINK_MOUNT_TAG 	"FSDevice"
 #define NAUTILUS_LINK_HOME_TAG 		"X-nautilus-home"
 
-static const char *
-get_tag (NautilusLinkType type)
-{
-	switch (type) {
-	default:
-		g_assert_not_reached ();
-		/* fall through */
-	case NAUTILUS_LINK_GENERIC:
-		return NAUTILUS_LINK_GENERIC_TAG;
-	case NAUTILUS_LINK_TRASH:
-		return NAUTILUS_LINK_TRASH_TAG;
-	case NAUTILUS_LINK_MOUNT:
-		return NAUTILUS_LINK_MOUNT_TAG;
-	case NAUTILUS_LINK_HOME:
-		return NAUTILUS_LINK_HOME_TAG;
-	}
-}
-
 static char *
 slurp_key_string (const char *uri,
 		  const char *keyname,
@@ -97,12 +79,13 @@ slurp_key_string (const char *uri,
 
 gboolean
 nautilus_link_desktop_file_local_create (const char        *directory_uri,
-					 const char        *name,
+					 const char        *base_name,
+					 const char        *display_name,
 					 const char        *image,
 					 const char        *target_uri,
 					 const GdkPoint    *point,
 					 int                screen,
-					 NautilusLinkType   type)
+					 gboolean           unique_filename)
 {
 	char *uri, *contents, *escaped_name;
 	GnomeDesktopItem *desktop_item;
@@ -110,23 +93,28 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 	NautilusFileChangesQueuePosition item;
 
 	g_return_val_if_fail (directory_uri != NULL, FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (base_name != NULL, FALSE);
+	g_return_val_if_fail (display_name != NULL, FALSE);
 	g_return_val_if_fail (target_uri != NULL, FALSE);
 
-	escaped_name = gnome_vfs_escape_string (name);
-	uri = g_strdup_printf ("%s/%s", directory_uri, escaped_name);
-	g_free (escaped_name);
+	if (unique_filename) {
+		uri = nautilus_ensure_unique_file_name (directory_uri,
+							base_name, ".desktop");
+	} else {
+		escaped_name = gnome_vfs_escape_string (base_name);
+		uri = g_strdup_printf ("%s/%s.desktop", directory_uri, escaped_name);
+		g_free (escaped_name);
+	}
 
 	contents = g_strdup_printf ("[Desktop Entry]\n"
 				    "Encoding=UTF-8\n"
 				    "Name=%s\n"
-				    "Type=%s\n"
+				    "Type=Link\n"
 				    "URL=%s\n"
 				    "%s%s\n",
-				    name,
-				    get_tag (type),
+				    display_name,
 				    target_uri,
-				    image != NULL ? "X-Nautilus-Icon=" : "",
+				    image != NULL ? "Icon=" : "",
 				    image != NULL ? image : "");
 	
 	desktop_item = gnome_desktop_item_new_from_string (uri,
@@ -172,25 +160,6 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 	return TRUE;
 }
 
-gboolean
-nautilus_link_desktop_file_local_set_icon (const char *uri,
-					   const char *icon_name)
-{
-	GnomeDesktopItem *desktop_file;
-	gboolean success;
-
-	desktop_file = gnome_desktop_item_new_from_uri (uri, 0, NULL);
-	if (desktop_file == NULL) {
-		return FALSE;
-	}
-
-	gnome_desktop_item_set_string (desktop_file, "X-Nautilus-Icon", icon_name);
-	success = gnome_desktop_item_save (desktop_file, NULL, FALSE, NULL);
-	gnome_desktop_item_unref (desktop_file);
-	
-	return success;
-}
-
 gboolean
 nautilus_link_desktop_file_local_set_text (const char *uri,
 					   const char *text)
@@ -244,56 +213,6 @@ nautilus_link_desktop_file_local_get_additional_text (const char *path)
 #endif
 }
 
-NautilusLinkType
-nautilus_link_desktop_file_local_get_link_type (const char *path)
-{
-	char *type;
-	NautilusLinkType retval;
-
-	type = slurp_key_string (path, "Type", FALSE);
-
-	if (type == NULL) {
-		return NAUTILUS_LINK_GENERIC;
-	}
-	if (strcmp (type, NAUTILUS_LINK_HOME_TAG) == 0) {
-		retval = NAUTILUS_LINK_HOME;
-	} else if (strcmp (type, NAUTILUS_LINK_MOUNT_TAG) == 0) {
-		retval = NAUTILUS_LINK_MOUNT;
-	} else if (strcmp (type, NAUTILUS_LINK_TRASH_TAG) == 0) {
-		retval = NAUTILUS_LINK_TRASH;
-	} else {
-		retval = NAUTILUS_LINK_GENERIC;
-	}
-
-	g_free (type);
-	return retval;
-}
-
-gboolean
-nautilus_link_desktop_file_local_is_utf8 (const char       *uri)
-{
-	char *contents;
-	int file_size;
-	gboolean retval;
-
-	if (eel_read_entire_file (uri,
-				  &file_size,
-				  &contents) != GNOME_VFS_OK) {
-		return FALSE;
-	}
-	
-	if (g_strstr_len (contents, file_size, "Encoding=UTF-8\n") != NULL) {
-		retval = TRUE;
-	} else {
-		retval = FALSE;
-	}
-
-	g_free (contents);
-	
-	return retval;
-}
-
-
 
 static char *
 nautilus_link_desktop_file_get_link_uri_from_desktop (GnomeDesktopItem *desktop_file)
@@ -341,6 +260,7 @@ nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop
 {
 	char *icon_uri;
 	const char *icon;
+	GnomeDesktopItemType desktop_type;
 
 	icon_uri = g_strdup (gnome_desktop_item_get_string (desktop_file, "X-Nautilus-Icon"));
 	if (icon_uri != NULL) {
@@ -348,7 +268,34 @@ nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop
 	}
 
 	icon = gnome_desktop_item_get_string (desktop_file, GNOME_DESKTOP_ITEM_ICON);
-	return g_strdup (icon);
+	if (icon != NULL) {
+		return g_strdup (icon);
+	}
+
+	desktop_type = gnome_desktop_item_get_entry_type (desktop_file);
+	switch (desktop_type) {
+	case GNOME_DESKTOP_ITEM_TYPE_APPLICATION:
+		return g_strdup ("gnome-fs-executable");
+
+	case GNOME_DESKTOP_ITEM_TYPE_LINK:
+		return g_strdup ("gnome-dev-symlink");
+
+	case GNOME_DESKTOP_ITEM_TYPE_FSDEVICE:
+		return g_strdup ("gnome-dev-harddisk");
+
+	case GNOME_DESKTOP_ITEM_TYPE_DIRECTORY:
+		return g_strdup ("gnome-fs-directory");
+
+	case GNOME_DESKTOP_ITEM_TYPE_SERVICE:
+	case GNOME_DESKTOP_ITEM_TYPE_SERVICE_TYPE:
+		return g_strdup ("gnome-fs-web");
+
+	default:
+		return g_strdup ("gnome-fs-regular");
+	}
+
+	g_assert_not_reached ();
+	return NULL;
 }
 
 char *
@@ -368,59 +315,38 @@ nautilus_link_desktop_file_local_get_link_uri (const char *uri)
 	return retval;
 }
 
-char *
-nautilus_link_desktop_file_get_link_uri_given_file_contents (const char *uri,
-							     const char *link_file_contents,
-							     int         link_file_size)
+void
+nautilus_link_desktop_file_get_link_info_given_file_contents (const char        *file_contents,
+							      int                link_file_size,
+							      char             **uri,
+							      char             **name,
+							      char             **icon,
+							      gulong            *drive_id,
+							      gulong            *volume_id)
 {
 	GnomeDesktopItem *desktop_file;
-	char *retval;
+	const char *id;
 
-	desktop_file = gnome_desktop_item_new_from_string (uri, link_file_contents, link_file_size, 0, NULL);
+	desktop_file = gnome_desktop_item_new_from_string (NULL, file_contents, link_file_size, 0, NULL);
 	if (desktop_file == NULL) {
-		return NULL; 
+		return; 
 	}
-	retval = nautilus_link_desktop_file_get_link_uri_from_desktop (desktop_file);
+	
+	*uri = nautilus_link_desktop_file_get_link_uri_from_desktop (desktop_file);
+	*name = nautilus_link_desktop_file_get_link_name_from_desktop (desktop_file);
+	*icon = nautilus_link_desktop_file_get_link_icon_from_desktop (desktop_file);
 
-	gnome_desktop_item_unref (desktop_file);
-	return retval;
-}
-
-char *
-nautilus_link_desktop_file_get_link_name_given_file_contents (const char *uri,
-							      const char *link_file_contents,
-							      int         link_file_size)
-{
-	GnomeDesktopItem *desktop_file;
-	char *retval;
-
-	desktop_file = gnome_desktop_item_new_from_string (uri, link_file_contents, link_file_size, 0, NULL);
-	if (desktop_file == NULL) {
-		return NULL; 
+	id = gnome_desktop_item_get_string (desktop_file, "X-Gnome-Volume");
+	if (id != NULL) {
+		*volume_id = atol (id);
 	}
-	retval = nautilus_link_desktop_file_get_link_name_from_desktop (desktop_file);
 
-	gnome_desktop_item_unref (desktop_file);
-	return retval;
-}
-
-
-char *
-nautilus_link_desktop_file_get_link_icon_given_file_contents (const char *uri,
-							      const char *link_file_contents,
-							      int         link_file_size)
-{
-	GnomeDesktopItem *desktop_file;
-	char *retval;
-
-	desktop_file = gnome_desktop_item_new_from_string (uri, link_file_contents, link_file_size, 0, NULL);
-	if (desktop_file == NULL) {
-		return NULL; 
+	id = gnome_desktop_item_get_string (desktop_file, "X-Gnome-Drive");
+	if (id != NULL) {
+		*drive_id = atol (id);
 	}
-	retval = nautilus_link_desktop_file_get_link_icon_from_desktop (desktop_file);
-
+	
 	gnome_desktop_item_unref (desktop_file);
-	return retval;
 }
 
 
