@@ -47,7 +47,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkstock.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-macros.h>
@@ -106,8 +106,6 @@ struct FMIconViewDetails
 
 	guint react_to_icon_change_idle_id;
 
-	gboolean loading;
-
 	const SortCriterion *sort;
 	gboolean sort_reversed;
 
@@ -289,20 +287,12 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 	scale_string = nautilus_file_get_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE, "1");
 	scale_good = sscanf
-		(scale_string, " %lf %c",
-		 &position->scale_x, &c) == 1;
-	if (scale_good) {
-		position->scale_y = position->scale_x;
-	} else {
-		scale_good = sscanf
-			(scale_string, " %lf %lf %c",
-			 &position->scale_x,
-			 &position->scale_y, &c) == 2;
-		if (!scale_good) {
-			position->scale_x = 1.0;
-			position->scale_y = 1.0;
-		}
+		(scale_string, " %lf",
+		 &position->scale) == 1;
+	if (!scale_good) {
+		position->scale = 1.0;
 	}
+
 	g_free (scale_string);
 
 	setlocale (LC_NUMERIC, locale);
@@ -518,6 +508,28 @@ fm_icon_view_remove_file (FMDirectoryView *view, NautilusFile *file, NautilusDir
 	}
 }
 
+static gboolean
+file_has_lazy_position (FMDirectoryView *view,
+			NautilusFile *file)
+{
+	gboolean lazy_position;
+
+	/* For volumes (i.e. cdrom icon) we use lazy positioning so that when
+	 * an old cdrom gets re-mounted in a place that now has another
+	 * icon we don't overlap that one. We don't do this in general though,
+	 * as it can cause icons moving around.
+	 */
+	lazy_position = nautilus_file_has_volume (file);
+	if (lazy_position && fm_directory_view_get_loading (view)) {
+		/* if volumes are loaded during directory load, don't mark them
+		 * as lazy. This is wrong for files that were mounted during user
+		 * log-off, but it is right for files that were mounted during login. */
+		lazy_position = FALSE;
+	}
+
+	return lazy_position;
+}
+
 static void
 fm_icon_view_add_file (FMDirectoryView *view, NautilusFile *file, NautilusDirectory *directory)
 {
@@ -535,13 +547,13 @@ fm_icon_view_add_file (FMDirectoryView *view, NautilusFile *file, NautilusDirect
 	}
 
 	/* Reset scroll region for the first icon added when loading a directory. */
-	if (icon_view->details->loading && nautilus_icon_container_is_empty (icon_container)) {
+	if (fm_directory_view_get_loading (view) && nautilus_icon_container_is_empty (icon_container)) {
 		nautilus_icon_container_reset_scroll_region (icon_container);
 	}
-	
+
 	if (nautilus_icon_container_add (icon_container,
 					 NAUTILUS_ICON_CONTAINER_ICON_DATA (file),
-					 NAUTILUS_IS_DESKTOP_ICON_FILE (file))) {
+					 file_has_lazy_position (view, file)))  {
 		nautilus_file_ref (file);
 	}
 }
@@ -2065,7 +2077,7 @@ fm_icon_view_screen_changed (GtkWidget *widget,
 			} else {
 				if (nautilus_icon_container_add (icon_container,
 								 NAUTILUS_ICON_CONTAINER_ICON_DATA (file),
-								 NAUTILUS_IS_DESKTOP_ICON_FILE (file))) {
+								 file_has_lazy_position (view, file))) {
 					nautilus_file_ref (file);
 				}
 			}
@@ -2136,7 +2148,7 @@ icon_position_changed_callback (NautilusIconContainer *container,
 				FMIconView *icon_view)
 {
 	char *position_string;
-	char *scale_string, *scale_string_x, *scale_string_y;
+	char *scale_string;
 	char *locale;
 
 	g_assert (FM_IS_ICON_VIEW (icon_view));
@@ -2179,16 +2191,7 @@ icon_position_changed_callback (NautilusIconContainer *container,
 	 * %.2f is not a good format for the scale factor. We'd like it to
 	 * say "2" or "2x" instead of "2.00".
 	 */
-	scale_string_x = g_strdup_printf ("%.2f", position->scale_x);
-	scale_string_y = g_strdup_printf ("%.2f", position->scale_y);
-	if (strcmp (scale_string_x, scale_string_y) == 0) {
-		scale_string = scale_string_x;
-		g_free (scale_string_y);
-	} else {
-		scale_string = g_strconcat (scale_string_x, ",", scale_string_y, NULL);
-		g_free (scale_string_x);
-		g_free (scale_string_y);
-	}
+	scale_string = g_strdup_printf ("%.2f", position->scale);
 	nautilus_file_set_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_SCALE,
 		 "1.00", scale_string);
@@ -2214,7 +2217,7 @@ fm_icon_view_icon_text_changed_callback (NautilusIconContainer *container,
 	if (new_name[0] == '\0') {
 		return;
 	}
-	fm_rename_file (file, new_name);
+	fm_rename_file (file, new_name, NULL, NULL);
 }
 
 static char *
